==> 0.0.2.tar.gz: lib/pages/conversation.dart <==
import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:timeago/timeago.dart' as time_ago;

import 'package:bizdoc/gen_l10n/app_localizations.dart';
import 'package:bizdoc/core/hub.dart';
import 'package:bizdoc/services/chat.dart';
import 'package:bizdoc/models/contracts.dart';
import 'package:bizdoc/models/models.dart';
import 'package:bizdoc/models/profile.dart';

class ConversationWidget extends StatefulWidget {
  final ContactInfo chatWith;

  const ConversationWidget({Key? key, required this.chatWith})
      : super(key: key);
  @override
  _ConversationWidgetState createState() => _ConversationWidgetState();
}

class _ConversationWidgetState extends State<ConversationWidget> {
  final _items = <Chat>[];
  late Future _prepare;
  final _controller = TextEditingController();

  final _typing = StreamController<bool>.broadcast();

  @override
  void initState() {
    _prepare = _retrieve();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<ChatContractBase?>(
        stream: hub.events
            .where((event) =>
                event is NewChat && event.contactId == widget.chatWith.id)
            .cast<NewChat>(),
        builder: (context, snaphshot) {
          if (snaphshot.hasData) {
            if (snaphshot.data is NewChat) {
              _items.add((snaphshot.data as NewChat).model);
            } else {
              if (snaphshot.data is ChatSeen) {
                widget.chatWith.watermark = (snaphshot.data as ChatSeen).time;
              } else {
                if (snaphshot.data is ChatTyping) {
                  _typing.add(true);
                  Future.delayed(
                      const Duration(seconds: 2), () => _typing.add(false));
                }
              }
            }
          }
          return Column(
            children: [
              Consumer(builder: (context, value, child) {
                return Column(children: [
                  Text(
                    widget.chatWith.name,
                    style: Theme.of(context).textTheme.headline2,
                  ),
                  Scrollable(
                    viewportBuilder: (context, position) => FutureBuilder(
                      future: _prepare,
                      builder: (context, snapshot) {
                        if (snapshot.hasData) {
                          return ListView.builder(
                            itemCount: _items.length,
                            itemBuilder: (context, index) {
                              final item = _items.elementAt(index);
                              return ListTile(
                                leading: item.image != null
                                    ? Image.memory(base64Decode(item.image!))
                                    : null,
                                title: Text(item.text ?? ''),
                                subtitle: Text(time_ago.format(item.time)),
                              );
                            },
                          );
                        }
                        return const CircularProgressIndicator();
                      },
                    ),
                  )
                ]);
              }),
              StreamBuilder<bool>(
                  stream: _typing.stream,
                  builder: (context, snapshot) {
                    if (snaphshot.data as bool) {
                      return Text(AppLocalizations.of(context)!.typing);
                    }
                    return Container();
                  }),
              TextField(
                controller: _controller,
                onTap: _seen,
                onChanged: (value) {
                  chat.typing(widget.chatWith.id);
                },
                onSubmitted: _send,
              ),
            ],
          );
        });
  }

  void _seen() async {
    if (widget.chatWith.count > 0) {
      await chat.seen(widget.chatWith.id);
      widget.chatWith.count = 0;
    }
  }

  Future _retrieve({DateTime? before}) async {
    try {
      _items
          .addAll(await chat.conversation(widget.chatWith.id, before: before));
    } catch (e) {
      _err();
    }
  }

  void _err() {
    ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text(AppLocalizations.of(context)!.err)));
  }

  Future<void> _send(String value) async {
    try {
      final model = await chat.chat(widget.chatWith.id, text: value);
      _items.add(model);
      _controller.clear();
    } catch (e) {
      _err();
    }
  }
}

==> 0.0.3.tar.gz: lib/pages/conversation.dart <==
import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:timeago/timeago.dart' as time_ago;

import 'package:bizdoc/gen_l10n/app_localizations.dart';
import 'package:bizdoc/core/hub.dart';
import 'package:bizdoc/services/chat.dart';
import 'package:bizdoc/models/contracts.dart';
import 'package:bizdoc/models/chat.dart';
import 'package:bizdoc/models/profile.dart';

class ConversationWidget extends StatefulWidget {
  final ContactInfo chatWith;

  const ConversationWidget({Key? key, required this.chatWith})
      : super(key: key);
  @override
  _ConversationWidgetState createState() => _ConversationWidgetState();
}

class _ConversationWidgetState extends State<ConversationWidget> {
  final _items = <Chat>[];
  late Future _prepare;
  final _controller = TextEditingController();

  final _typing = StreamController<bool>.broadcast();

  @override
  void initState() {
    _prepare = _retrieve();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<ChatContractBase?>(
        stream: hub.events
            .where((event) =>
                event is NewChat && event.contactId == widget.chatWith.id)
            .cast<NewChat>(),
        builder: (context, snaphshot) {
          if (snaphshot.hasData) {
            if (snaphshot.data is NewChat) {
              _items.add((snaphshot.data as NewChat).model);
            } else {
              if (snaphshot.data is ChatSeen) {
                widget.chatWith.watermark = (snaphshot.data as ChatSeen).time;
              } else {
                if (snaphshot.data is ChatTyping) {
                  _typing.add(true);
                  Future.delayed(
                      const Duration(seconds: 2), () => _typing.add(false));
                }
              }
            }
          }
          return Column(
            children: [
              Consumer(builder: (context, value, child) {
                return Column(children: [
                  Text(
                    widget.chatWith.name,
                    style: Theme.of(context).textTheme.headline2,
                  ),
                  Scrollable(
                    viewportBuilder: (context, position) => FutureBuilder(
                      future: _prepare,
                      builder: (context, snapshot) {
                        if (snapshot.hasData) {
                          return ListView.builder(
                            itemCount: _items.length,
                            itemBuilder: (context, index) {
                              final item = _items.elementAt(index);
                              return ListTile(
                                leading: item.image != null
                                    ? Image.memory(base64Decode(item.image!))
                                    : null,
                                title: Text(item.text ?? ''),
                                subtitle: Text(time_ago.format(item.time)),
                              );
                            },
                          );
                        }
                        return const CircularProgressIndicator();
                      },
                    ),
                  )
                ]);
              }),
              StreamBuilder<bool>(
                  stream: _typing.stream,
                  builder: (context, snapshot) {
                    if (snaphshot.data as bool) {
                      return Text(AppLocalizations.of(context)!.typing);
                    }
                    return Container();
                  }),
              TextField(
                controller: _controller,
                onTap: _seen,
                onChanged: (value) {
                  chat.typing(widget.chatWith.id);
                },
                onSubmitted: _send,
              ),
            ],
          );
        });
  }

  void _seen() async {
    if (widget.chatWith.count > 0) {
      await chat.seen(widget.chatWith.id);
      widget.chatWith.count = 0;
    }
  }

  Future _retrieve({DateTime? before}) async {
    try {
      _items
          .addAll(await chat.conversation(widget.chatWith.id, before: before));
    } catch (e) {
      _err();
    }
  }

  void _err() {
    ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text(AppLocalizations.of(context)!.err)));
  }

  Future<void> _send(String value) async {
    try {
      final model = await chat.chat(widget.chatWith.id, text: value);
      _items.add(model);
      _controller.clear();
    } catch (e) {
      _err();
    }
  }
}
